# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .order import (
    OrderResource,
    AsyncOrderResource,
    OrderResourceWithRawResponse,
    AsyncOrderResourceWithRawResponse,
    OrderResourceWithStreamingResponse,
    AsyncOrderResourceWithStreamingResponse,
)
from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......pagination import SyncSinglePage, AsyncSinglePage
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.dlp.sensitivity_groups import level_create_params, level_update_params
from ......types.zero_trust.dlp.sensitivity_groups.level_get_response import LevelGetResponse
from ......types.zero_trust.dlp.sensitivity_groups.level_list_response import LevelListResponse
from ......types.zero_trust.dlp.sensitivity_groups.level_create_response import LevelCreateResponse
from ......types.zero_trust.dlp.sensitivity_groups.level_update_response import LevelUpdateResponse

__all__ = ["LevelsResource", "AsyncLevelsResource"]


class LevelsResource(SyncAPIResource):
    @cached_property
    def order(self) -> OrderResource:
        return OrderResource(self._client)

    @cached_property
    def with_raw_response(self) -> LevelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LevelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LevelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LevelsResourceWithStreamingResponse(self)

    def create(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LevelCreateResponse]:
        """
        Creates a sensitivity level in a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._post(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                },
                level_create_params.LevelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LevelCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LevelCreateResponse]], ResultWrapper[LevelCreateResponse]),
        )

    def update(
        self,
        sensitivity_level_id: str,
        *,
        account_id: str,
        sensitivity_group_id: str,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LevelUpdateResponse]:
        """
        Updates a sensitivity level in a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        if not sensitivity_level_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_level_id` but received {sensitivity_level_id!r}"
            )
        return self._put(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels/{sensitivity_level_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
                sensitivity_level_id=sensitivity_level_id,
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                level_update_params.LevelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LevelUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LevelUpdateResponse]], ResultWrapper[LevelUpdateResponse]),
        )

    def list(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[LevelListResponse]:
        """
        Lists sensitivity levels in a sensitivity group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            page=SyncSinglePage[LevelListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LevelListResponse,
        )

    def delete(
        self,
        sensitivity_level_id: str,
        *,
        account_id: str,
        sensitivity_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a sensitivity level from a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        if not sensitivity_level_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_level_id` but received {sensitivity_level_id!r}"
            )
        return self._delete(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels/{sensitivity_level_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
                sensitivity_level_id=sensitivity_level_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        sensitivity_level_id: str,
        *,
        account_id: str,
        sensitivity_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LevelGetResponse]:
        """
        Gets a sensitivity level from a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        if not sensitivity_level_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_level_id` but received {sensitivity_level_id!r}"
            )
        return self._get(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels/{sensitivity_level_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
                sensitivity_level_id=sensitivity_level_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LevelGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LevelGetResponse]], ResultWrapper[LevelGetResponse]),
        )


class AsyncLevelsResource(AsyncAPIResource):
    @cached_property
    def order(self) -> AsyncOrderResource:
        return AsyncOrderResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLevelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLevelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLevelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLevelsResourceWithStreamingResponse(self)

    async def create(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LevelCreateResponse]:
        """
        Creates a sensitivity level in a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return await self._post(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                },
                level_create_params.LevelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LevelCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LevelCreateResponse]], ResultWrapper[LevelCreateResponse]),
        )

    async def update(
        self,
        sensitivity_level_id: str,
        *,
        account_id: str,
        sensitivity_group_id: str,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LevelUpdateResponse]:
        """
        Updates a sensitivity level in a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        if not sensitivity_level_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_level_id` but received {sensitivity_level_id!r}"
            )
        return await self._put(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels/{sensitivity_level_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
                sensitivity_level_id=sensitivity_level_id,
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                level_update_params.LevelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LevelUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LevelUpdateResponse]], ResultWrapper[LevelUpdateResponse]),
        )

    def list(
        self,
        sensitivity_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LevelListResponse, AsyncSinglePage[LevelListResponse]]:
        """
        Lists sensitivity levels in a sensitivity group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
            ),
            page=AsyncSinglePage[LevelListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LevelListResponse,
        )

    async def delete(
        self,
        sensitivity_level_id: str,
        *,
        account_id: str,
        sensitivity_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a sensitivity level from a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        if not sensitivity_level_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_level_id` but received {sensitivity_level_id!r}"
            )
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels/{sensitivity_level_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
                sensitivity_level_id=sensitivity_level_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        sensitivity_level_id: str,
        *,
        account_id: str,
        sensitivity_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LevelGetResponse]:
        """
        Gets a sensitivity level from a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sensitivity_group_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_group_id` but received {sensitivity_group_id!r}"
            )
        if not sensitivity_level_id:
            raise ValueError(
                f"Expected a non-empty value for `sensitivity_level_id` but received {sensitivity_level_id!r}"
            )
        return await self._get(
            path_template(
                "/accounts/{account_id}/dlp/sensitivity_groups/{sensitivity_group_id}/levels/{sensitivity_level_id}",
                account_id=account_id,
                sensitivity_group_id=sensitivity_group_id,
                sensitivity_level_id=sensitivity_level_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LevelGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LevelGetResponse]], ResultWrapper[LevelGetResponse]),
        )


class LevelsResourceWithRawResponse:
    def __init__(self, levels: LevelsResource) -> None:
        self._levels = levels

        self.create = to_raw_response_wrapper(
            levels.create,
        )
        self.update = to_raw_response_wrapper(
            levels.update,
        )
        self.list = to_raw_response_wrapper(
            levels.list,
        )
        self.delete = to_raw_response_wrapper(
            levels.delete,
        )
        self.get = to_raw_response_wrapper(
            levels.get,
        )

    @cached_property
    def order(self) -> OrderResourceWithRawResponse:
        return OrderResourceWithRawResponse(self._levels.order)


class AsyncLevelsResourceWithRawResponse:
    def __init__(self, levels: AsyncLevelsResource) -> None:
        self._levels = levels

        self.create = async_to_raw_response_wrapper(
            levels.create,
        )
        self.update = async_to_raw_response_wrapper(
            levels.update,
        )
        self.list = async_to_raw_response_wrapper(
            levels.list,
        )
        self.delete = async_to_raw_response_wrapper(
            levels.delete,
        )
        self.get = async_to_raw_response_wrapper(
            levels.get,
        )

    @cached_property
    def order(self) -> AsyncOrderResourceWithRawResponse:
        return AsyncOrderResourceWithRawResponse(self._levels.order)


class LevelsResourceWithStreamingResponse:
    def __init__(self, levels: LevelsResource) -> None:
        self._levels = levels

        self.create = to_streamed_response_wrapper(
            levels.create,
        )
        self.update = to_streamed_response_wrapper(
            levels.update,
        )
        self.list = to_streamed_response_wrapper(
            levels.list,
        )
        self.delete = to_streamed_response_wrapper(
            levels.delete,
        )
        self.get = to_streamed_response_wrapper(
            levels.get,
        )

    @cached_property
    def order(self) -> OrderResourceWithStreamingResponse:
        return OrderResourceWithStreamingResponse(self._levels.order)


class AsyncLevelsResourceWithStreamingResponse:
    def __init__(self, levels: AsyncLevelsResource) -> None:
        self._levels = levels

        self.create = async_to_streamed_response_wrapper(
            levels.create,
        )
        self.update = async_to_streamed_response_wrapper(
            levels.update,
        )
        self.list = async_to_streamed_response_wrapper(
            levels.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            levels.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            levels.get,
        )

    @cached_property
    def order(self) -> AsyncOrderResourceWithStreamingResponse:
        return AsyncOrderResourceWithStreamingResponse(self._levels.order)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = ["PermissionListResponse", "PermissionListResponseItem"]


class PermissionListResponseItem(BaseModel):
    id: Optional[int] = None
    """The unique identifier for the indicator feed"""

    description: Optional[str] = None
    """The description of the example test"""

    is_attributable: Optional[bool] = None
    """Whether the indicator feed can be attributed to a provider"""

    is_downloadable: Optional[bool] = None
    """Whether the indicator feed can be downloaded"""

    is_public: Optional[bool] = None
    """Whether the indicator feed is exposed to customers"""

    name: Optional[str] = None
    """The name of the indicator feed"""


PermissionListResponse: TypeAlias = List[PermissionListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tags import (
    TagsResource,
    AsyncTagsResource,
    TagsResourceWithRawResponse,
    AsyncTagsResourceWithRawResponse,
    TagsResourceWithStreamingResponse,
    AsyncTagsResourceWithStreamingResponse,
)
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from .indicators import (
    IndicatorsResource,
    AsyncIndicatorsResource,
    IndicatorsResourceWithRawResponse,
    AsyncIndicatorsResourceWithRawResponse,
    IndicatorsResourceWithStreamingResponse,
    AsyncIndicatorsResourceWithStreamingResponse,
)

__all__ = [
    "CategoriesResource",
    "AsyncCategoriesResource",
    "CategoriesResourceWithRawResponse",
    "AsyncCategoriesResourceWithRawResponse",
    "CategoriesResourceWithStreamingResponse",
    "AsyncCategoriesResourceWithStreamingResponse",
    "IndicatorsResource",
    "AsyncIndicatorsResource",
    "IndicatorsResourceWithRawResponse",
    "AsyncIndicatorsResourceWithRawResponse",
    "IndicatorsResourceWithStreamingResponse",
    "AsyncIndicatorsResourceWithStreamingResponse",
    "TagsResource",
    "AsyncTagsResource",
    "TagsResourceWithRawResponse",
    "AsyncTagsResourceWithRawResponse",
    "TagsResourceWithStreamingResponse",
    "AsyncTagsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RegistrationListParams"]


class RegistrationListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    cursor: str
    """
    Opaque token from a previous response's `result_info.cursor`. Pass this value to
    fetch the next page of results. Omit (or pass an empty string) for the first
    page.
    """

    direction: Literal["asc", "desc"]
    """Sort direction for results. Defaults to ascending order."""

    per_page: int
    """Number of items to return per page."""

    sort_by: Literal["registry_created_at", "registry_expires_at", "name"]
    """Column to sort results by.

    Defaults to registration date (`registry_created_at`) when omitted.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from .by_dataset import (
    ByDatasetResource,
    AsyncByDatasetResource,
    ByDatasetResourceWithRawResponse,
    AsyncByDatasetResourceWithRawResponse,
    ByDatasetResourceWithStreamingResponse,
    AsyncByDatasetResourceWithStreamingResponse,
)
from ......_types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_base_client import make_request_options
from ......types.cloudforce_one.threat_events.tags import indicator_list_params
from ......types.cloudforce_one.threat_events.tags.indicator_list_response import IndicatorListResponse

__all__ = ["IndicatorsResource", "AsyncIndicatorsResource"]


class IndicatorsResource(SyncAPIResource):
    @cached_property
    def by_dataset(self) -> ByDatasetResource:
        return ByDatasetResource(self._client)

    @cached_property
    def with_raw_response(self) -> IndicatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IndicatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IndicatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IndicatorsResourceWithStreamingResponse(self)

    def list(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        indicator_type: str | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        related_event: SequenceNotStr[str] | Omit = omit,
        search: Iterable[indicator_list_params.Search] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IndicatorListResponse:
        """Returns indicators associated with the provided tag UUID, with pagination.

        By
        default fans out across every indicator dataset the account can read; pass
        datasetIds to scope to specific datasets.

        Args:
          account_id: Account ID.

          tag_uuid: Tag UUID.

          dataset_ids: Dataset UUIDs to scope to (repeat the param for multiple), or 'all' / '\\**' for
              every readable indicator dataset. Omit to search all readable datasets.

          related_event: Filter indicators by related event UUID(s). Multiple UUIDs can be provided by
              repeating the parameter.

          search: Structured search as a JSON array of {field, op, value} objects. Searchable
              fields: value, indicatorType. Multiple conditions are AND'd together. Max 10
              conditions per request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}/indicators",
                account_id=account_id,
                tag_uuid=tag_uuid,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dataset_ids": dataset_ids,
                        "indicator_type": indicator_type,
                        "page": page,
                        "page_size": page_size,
                        "related_event": related_event,
                        "search": search,
                    },
                    indicator_list_params.IndicatorListParams,
                ),
            ),
            cast_to=IndicatorListResponse,
        )


class AsyncIndicatorsResource(AsyncAPIResource):
    @cached_property
    def by_dataset(self) -> AsyncByDatasetResource:
        return AsyncByDatasetResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncIndicatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIndicatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIndicatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIndicatorsResourceWithStreamingResponse(self)

    async def list(
        self,
        tag_uuid: str,
        *,
        account_id: str,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        indicator_type: str | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        related_event: SequenceNotStr[str] | Omit = omit,
        search: Iterable[indicator_list_params.Search] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IndicatorListResponse:
        """Returns indicators associated with the provided tag UUID, with pagination.

        By
        default fans out across every indicator dataset the account can read; pass
        datasetIds to scope to specific datasets.

        Args:
          account_id: Account ID.

          tag_uuid: Tag UUID.

          dataset_ids: Dataset UUIDs to scope to (repeat the param for multiple), or 'all' / '\\**' for
              every readable indicator dataset. Omit to search all readable datasets.

          related_event: Filter indicators by related event UUID(s). Multiple UUIDs can be provided by
              repeating the parameter.

          search: Structured search as a JSON array of {field, op, value} objects. Searchable
              fields: value, indicatorType. Multiple conditions are AND'd together. Max 10
              conditions per request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tag_uuid:
            raise ValueError(f"Expected a non-empty value for `tag_uuid` but received {tag_uuid!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/tags/{tag_uuid}/indicators",
                account_id=account_id,
                tag_uuid=tag_uuid,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "dataset_ids": dataset_ids,
                        "indicator_type": indicator_type,
                        "page": page,
                        "page_size": page_size,
                        "related_event": related_event,
                        "search": search,
                    },
                    indicator_list_params.IndicatorListParams,
                ),
            ),
            cast_to=IndicatorListResponse,
        )


class IndicatorsResourceWithRawResponse:
    def __init__(self, indicators: IndicatorsResource) -> None:
        self._indicators = indicators

        self.list = to_raw_response_wrapper(
            indicators.list,
        )

    @cached_property
    def by_dataset(self) -> ByDatasetResourceWithRawResponse:
        return ByDatasetResourceWithRawResponse(self._indicators.by_dataset)


class AsyncIndicatorsResourceWithRawResponse:
    def __init__(self, indicators: AsyncIndicatorsResource) -> None:
        self._indicators = indicators

        self.list = async_to_raw_response_wrapper(
            indicators.list,
        )

    @cached_property
    def by_dataset(self) -> AsyncByDatasetResourceWithRawResponse:
        return AsyncByDatasetResourceWithRawResponse(self._indicators.by_dataset)


class IndicatorsResourceWithStreamingResponse:
    def __init__(self, indicators: IndicatorsResource) -> None:
        self._indicators = indicators

        self.list = to_streamed_response_wrapper(
            indicators.list,
        )

    @cached_property
    def by_dataset(self) -> ByDatasetResourceWithStreamingResponse:
        return ByDatasetResourceWithStreamingResponse(self._indicators.by_dataset)


class AsyncIndicatorsResourceWithStreamingResponse:
    def __init__(self, indicators: AsyncIndicatorsResource) -> None:
        self._indicators = indicators

        self.list = async_to_streamed_response_wrapper(
            indicators.list,
        )

    @cached_property
    def by_dataset(self) -> AsyncByDatasetResourceWithStreamingResponse:
        return AsyncByDatasetResourceWithStreamingResponse(self._indicators.by_dataset)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["TagDeleteResponse"]


class TagDeleteResponse(BaseModel):
    uuid: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EmailRoutingEditParams"]


class EmailRoutingEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    enabled: Literal[True, False]
    """State of your zone Email Routing settings.

    No-op on this endpoint - use `POST`/`DELETE /zones/{zone_id}/email/routing/dns`.
    """

    skip_wizard: Literal[True, False]
    """Flag to check if the user skipped the configuration wizard."""

    support_subaddress: Literal[True, False]
    """
    Whether subaddressing (plus-addressing) is honored when matching incoming mail
    against routing rules.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["RampType"]

RampType: TypeAlias = Literal["gre", "gre_interconnect", "mpls_interconnect", "mconn", "ipsec"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._utils import PropertyInfo

__all__ = ["StatusEditParams", "Variant0", "Variant1", "Variant2", "Variant3", "Variant3From"]


class Variant0(TypedDict, total=False):
    account_id: Required[str]

    workflow_name: Required[str]

    status: Required[Literal["pause"]]


class Variant1(TypedDict, total=False):
    account_id: Required[str]

    workflow_name: Required[str]

    status: Required[Literal["resume"]]


class Variant2(TypedDict, total=False):
    account_id: Required[str]

    workflow_name: Required[str]

    status: Required[Literal["terminate"]]

    rollback: bool
    """Run rollback before terminating."""


class Variant3(TypedDict, total=False):
    account_id: Required[str]

    workflow_name: Required[str]

    status: Required[Literal["restart"]]

    from_: Annotated[Variant3From, PropertyInfo(alias="from")]
    """Step to restart from."""


class Variant3From(TypedDict, total=False):
    """Step to restart from."""

    name: Required[str]

    count: int

    type: Literal["do", "sleep", "waitForEvent"]


StatusEditParams: TypeAlias = Union[Variant0, Variant1, Variant2, Variant3]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .content import (
    ContentResource,
    AsyncContentResource,
    ContentResourceWithRawResponse,
    AsyncContentResourceWithRawResponse,
    ContentResourceWithStreamingResponse,
    AsyncContentResourceWithStreamingResponse,
)
from .exports import (
    ExportsResource,
    AsyncExportsResource,
    ExportsResourceWithRawResponse,
    AsyncExportsResourceWithRawResponse,
    ExportsResourceWithStreamingResponse,
    AsyncExportsResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from .findings.findings import (
    FindingsResource,
    AsyncFindingsResource,
    FindingsResourceWithRawResponse,
    AsyncFindingsResourceWithRawResponse,
    FindingsResourceWithStreamingResponse,
    AsyncFindingsResourceWithStreamingResponse,
)
from .webhooks.webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)
from .remediations.remediations import (
    RemediationsResource,
    AsyncRemediationsResource,
    RemediationsResourceWithRawResponse,
    AsyncRemediationsResourceWithRawResponse,
    RemediationsResourceWithStreamingResponse,
    AsyncRemediationsResourceWithStreamingResponse,
)
from .finding_types.finding_types import (
    FindingTypesResource,
    AsyncFindingTypesResource,
    FindingTypesResourceWithRawResponse,
    AsyncFindingTypesResourceWithRawResponse,
    FindingTypesResourceWithStreamingResponse,
    AsyncFindingTypesResourceWithStreamingResponse,
)

__all__ = ["PostureResource", "AsyncPostureResource"]


class PostureResource(SyncAPIResource):
    @cached_property
    def findings(self) -> FindingsResource:
        return FindingsResource(self._client)

    @cached_property
    def exports(self) -> ExportsResource:
        return ExportsResource(self._client)

    @cached_property
    def finding_types(self) -> FindingTypesResource:
        return FindingTypesResource(self._client)

    @cached_property
    def content(self) -> ContentResource:
        return ContentResource(self._client)

    @cached_property
    def remediations(self) -> RemediationsResource:
        return RemediationsResource(self._client)

    @cached_property
    def webhooks(self) -> WebhooksResource:
        return WebhooksResource(self._client)

    @cached_property
    def with_raw_response(self) -> PostureResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PostureResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PostureResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PostureResourceWithStreamingResponse(self)


class AsyncPostureResource(AsyncAPIResource):
    @cached_property
    def findings(self) -> AsyncFindingsResource:
        return AsyncFindingsResource(self._client)

    @cached_property
    def exports(self) -> AsyncExportsResource:
        return AsyncExportsResource(self._client)

    @cached_property
    def finding_types(self) -> AsyncFindingTypesResource:
        return AsyncFindingTypesResource(self._client)

    @cached_property
    def content(self) -> AsyncContentResource:
        return AsyncContentResource(self._client)

    @cached_property
    def remediations(self) -> AsyncRemediationsResource:
        return AsyncRemediationsResource(self._client)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        return AsyncWebhooksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPostureResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPostureResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPostureResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPostureResourceWithStreamingResponse(self)


class PostureResourceWithRawResponse:
    def __init__(self, posture: PostureResource) -> None:
        self._posture = posture

    @cached_property
    def findings(self) -> FindingsResourceWithRawResponse:
        return FindingsResourceWithRawResponse(self._posture.findings)

    @cached_property
    def exports(self) -> ExportsResourceWithRawResponse:
        return ExportsResourceWithRawResponse(self._posture.exports)

    @cached_property
    def finding_types(self) -> FindingTypesResourceWithRawResponse:
        return FindingTypesResourceWithRawResponse(self._posture.finding_types)

    @cached_property
    def content(self) -> ContentResourceWithRawResponse:
        return ContentResourceWithRawResponse(self._posture.content)

    @cached_property
    def remediations(self) -> RemediationsResourceWithRawResponse:
        return RemediationsResourceWithRawResponse(self._posture.remediations)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithRawResponse:
        return WebhooksResourceWithRawResponse(self._posture.webhooks)


class AsyncPostureResourceWithRawResponse:
    def __init__(self, posture: AsyncPostureResource) -> None:
        self._posture = posture

    @cached_property
    def findings(self) -> AsyncFindingsResourceWithRawResponse:
        return AsyncFindingsResourceWithRawResponse(self._posture.findings)

    @cached_property
    def exports(self) -> AsyncExportsResourceWithRawResponse:
        return AsyncExportsResourceWithRawResponse(self._posture.exports)

    @cached_property
    def finding_types(self) -> AsyncFindingTypesResourceWithRawResponse:
        return AsyncFindingTypesResourceWithRawResponse(self._posture.finding_types)

    @cached_property
    def content(self) -> AsyncContentResourceWithRawResponse:
        return AsyncContentResourceWithRawResponse(self._posture.content)

    @cached_property
    def remediations(self) -> AsyncRemediationsResourceWithRawResponse:
        return AsyncRemediationsResourceWithRawResponse(self._posture.remediations)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithRawResponse:
        return AsyncWebhooksResourceWithRawResponse(self._posture.webhooks)


class PostureResourceWithStreamingResponse:
    def __init__(self, posture: PostureResource) -> None:
        self._posture = posture

    @cached_property
    def findings(self) -> FindingsResourceWithStreamingResponse:
        return FindingsResourceWithStreamingResponse(self._posture.findings)

    @cached_property
    def exports(self) -> ExportsResourceWithStreamingResponse:
        return ExportsResourceWithStreamingResponse(self._posture.exports)

    @cached_property
    def finding_types(self) -> FindingTypesResourceWithStreamingResponse:
        return FindingTypesResourceWithStreamingResponse(self._posture.finding_types)

    @cached_property
    def content(self) -> ContentResourceWithStreamingResponse:
        return ContentResourceWithStreamingResponse(self._posture.content)

    @cached_property
    def remediations(self) -> RemediationsResourceWithStreamingResponse:
        return RemediationsResourceWithStreamingResponse(self._posture.remediations)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithStreamingResponse:
        return WebhooksResourceWithStreamingResponse(self._posture.webhooks)


class AsyncPostureResourceWithStreamingResponse:
    def __init__(self, posture: AsyncPostureResource) -> None:
        self._posture = posture

    @cached_property
    def findings(self) -> AsyncFindingsResourceWithStreamingResponse:
        return AsyncFindingsResourceWithStreamingResponse(self._posture.findings)

    @cached_property
    def exports(self) -> AsyncExportsResourceWithStreamingResponse:
        return AsyncExportsResourceWithStreamingResponse(self._posture.exports)

    @cached_property
    def finding_types(self) -> AsyncFindingTypesResourceWithStreamingResponse:
        return AsyncFindingTypesResourceWithStreamingResponse(self._posture.finding_types)

    @cached_property
    def content(self) -> AsyncContentResourceWithStreamingResponse:
        return AsyncContentResourceWithStreamingResponse(self._posture.content)

    @cached_property
    def remediations(self) -> AsyncRemediationsResourceWithStreamingResponse:
        return AsyncRemediationsResourceWithStreamingResponse(self._posture.remediations)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithStreamingResponse:
        return AsyncWebhooksResourceWithStreamingResponse(self._posture.webhooks)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["RegistrarSandboxCheckParams"]


class RegistrarSandboxCheckParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    domains: Required[SequenceNotStr[str]]
    """List of fully qualified domain names (FQDNs) to check for availability.

    Each domain must include the extension.

    - Minimum: 1 domain
    - Maximum: 20 domains per request
    - Domains on unsupported extensions are returned with `registrable: false` and a
      `reason` field
    - Malformed domain names (e.g., missing extension) may be omitted from the
      response
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ......_types import SequenceNotStr
from ......_utils import PropertyInfo

__all__ = ["InstanceListParams"]


class InstanceListParams(TypedDict, total=False):
    account_id: Required[str]

    archived: bool
    """Archived"""

    asset_ids: SequenceNotStr[str]
    """Filter finding instances by an array of asset IDs.

    Supports multiple comma-separated values.
    """

    cursor: str
    """A cursor for pagination.

    Obtained from the `result_info.cursor` field of a previous response.
    """

    direction: Literal["asc", "desc"]
    """Direction to order results."""

    finding_instance_ids: SequenceNotStr[str]
    """Filter finding instances by an array of finding instance IDs.

    Supports multiple comma-separated values.
    """

    max_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view findings that occurred on or before the affliction date.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    min_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view findings that occurred on or after the affliction date.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    order: Literal["affliction_date", "asset.name", "remediation.status"]
    """
    Which field to use when ordering the Finding's instances. When ordering by
    'remediation.status', only the most recent non-stale remediation job is
    considered. Stale jobs (created before the instance's affliction_date) are
    treated as having no status for ordering purposes.
    """

    page: int
    """A page number within the paginated result set."""

    per_page: int
    """Number of results to return per page."""

    remediation_statuses: List[Literal["none", "pending", "processing", "validating", "completed", "failed"]]
    """Filter finding instances by most recent remediation job status.

    Supports multiple comma-separated values. Use 'none' to filter instances with no
    remediation jobs or instances where the most recent job is stale. Note: Stale
    jobs (created before the instance's affliction_date) are ignored for filtering
    purposes, but are still included in the 'remediations' array with stale=true.
    """

    search: str
    """A search term."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .prefix_binding_edit_params import PrefixBindingEditParams as PrefixBindingEditParams
from .prefix_binding_list_params import PrefixBindingListParams as PrefixBindingListParams
from .prefix_binding_get_response import PrefixBindingGetResponse as PrefixBindingGetResponse
from .prefix_binding_create_params import PrefixBindingCreateParams as PrefixBindingCreateParams
from .prefix_binding_edit_response import PrefixBindingEditResponse as PrefixBindingEditResponse
from .prefix_binding_list_response import PrefixBindingListResponse as PrefixBindingListResponse
from .prefix_binding_create_response import PrefixBindingCreateResponse as PrefixBindingCreateResponse
from .prefix_binding_delete_response import PrefixBindingDeleteResponse as PrefixBindingDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.zero_trust.casb import integration_list_params, integration_create_params, integration_update_params
from ....types.zero_trust.casb.integration_get_response import IntegrationGetResponse
from ....types.zero_trust.casb.integration_pause_response import IntegrationPauseResponse
from ....types.zero_trust.casb.integration_create_response import IntegrationCreateResponse
from ....types.zero_trust.casb.integration_resume_response import IntegrationResumeResponse
from ....types.zero_trust.casb.integration_update_response import IntegrationUpdateResponse

__all__ = ["IntegrationsResource", "AsyncIntegrationsResource"]


class IntegrationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IntegrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IntegrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IntegrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IntegrationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        application: Literal[
            "ANTHROPIC",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SLACK",
        ],
        credentials: Dict[str, object],
        name: str,
        auth_method: Optional[str] | Omit = omit,
        dlp_profiles: SequenceNotStr[str] | Omit = omit,
        permissions: SequenceNotStr[str] | Omit = omit,
        use_cases: List[Literal["casb", "ces", "auto_remediation"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationCreateResponse:
        """Creates a new integration for the specified application.

        Integration creation
        with OAuth is not supported by API at the moment. For other auth methods, use
        `GET /v2/applications/{application_id}/credential-guide` to see the required
        credential structure and example payloads for each vendor.

        Args:
          application: Vendor/application slug (e.g., GOOGLE_WORKSPACE).

              - `ANTHROPIC` - ANTHROPIC
              - `BITBUCKET` - BITBUCKET
              - `BOX` - BOX
              - `CONFLUENCE` - CONFLUENCE
              - `DROPBOX` - DROPBOX
              - `GITHUB` - GITHUB
              - `GOOGLE_CLOUD_PLATFORM` - GOOGLE_CLOUD_PLATFORM
              - `GOOGLE_WORKSPACE` - GOOGLE_WORKSPACE
              - `JIRA` - JIRA
              - `MICROSOFT_INTERNAL` - MICROSOFT_INTERNAL
              - `OPENAI` - OPENAI
              - `SALESFORCE` - SALESFORCE
              - `SLACK` - SLACK

          credentials: Credentials for the integration.

          name: Name of the integration.

          auth_method: Authentication method slug (uses default if omitted).

          dlp_profiles: List of DLP profile IDs to associate.

          permissions: List of permission scopes (uses policy defaults if empty).

          use_cases: List of use case or feature slugs to enroll (e.g., ['casb', 'ces',
              'auto_remediation']).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/one/integrations", account_id=account_id),
            body=maybe_transform(
                {
                    "application": application,
                    "credentials": credentials,
                    "name": name,
                    "auth_method": auth_method,
                    "dlp_profiles": dlp_profiles,
                    "permissions": permissions,
                    "use_cases": use_cases,
                },
                integration_create_params.IntegrationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationCreateResponse,
        )

    def update(
        self,
        id: str,
        *,
        account_id: str,
        credentials: Dict[str, object] | Omit = omit,
        dlp_profiles: SequenceNotStr[str] | Omit = omit,
        name: str | Omit = omit,
        permissions: SequenceNotStr[str] | Omit = omit,
        use_cases: List[Literal["casb", "ces", "auto_remediation"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationUpdateResponse:
        """
        Updates an integration's name, permissions, DLP profiles, use cases, or
        credentials.

        Args:
          credentials: Partial credential fields to merge with existing.

          dlp_profiles: List of DLP profile IDs to associate with the integration.

          name: Name of the integration.

          permissions: List of permission scopes granted to the integration.

          use_cases: List of use case or feature slugs to enroll (e.g., ['casb', 'ces',
              'auto_remediation']).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/one/integrations/{id}", account_id=account_id, id=id),
            body=maybe_transform(
                {
                    "credentials": credentials,
                    "dlp_profiles": dlp_profiles,
                    "name": name,
                    "permissions": permissions,
                    "use_cases": use_cases,
                },
                integration_update_params.IntegrationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationUpdateResponse,
        )

    def list(
        self,
        *,
        account_id: str,
        application: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        dlp_enabled: bool | Omit = omit,
        order: Literal["application", "created", "name", "status"] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        search: str | Omit = omit,
        status: Literal["Healthy", "Initializing", "Offline", "Unhealthy"] | Omit = omit,
        use_cases: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Returns a paginated list of integrations for the account.

        Args:
          application: Filter by application/vendor (e.g., GOOGLE_WORKSPACE, MICROSOFT_INTERNAL).

          direction: Direction to order results.

          dlp_enabled: Filter by DLP enabled status (true/false).

          order: Field to order results by.

          page: Page number within the paginated result set.

          page_size: Number of results per page.

          search: Search integrations by name or application.

          status: Filter by integration status.

          use_cases: Filter by enabled use cases (e.g., casb, ces). Matches integrations enrolled in
              any of the specified values. Can be specified multiple times.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/one/integrations", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "application": application,
                        "direction": direction,
                        "dlp_enabled": dlp_enabled,
                        "order": order,
                        "page": page,
                        "page_size": page_size,
                        "search": search,
                        "status": status,
                        "use_cases": use_cases,
                    },
                    integration_list_params.IntegrationListParams,
                ),
            ),
            cast_to=object,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an integration by soft-deleting it.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/accounts/{account_id}/one/integrations/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationGetResponse:
        """
        Returns full integration details including use cases and permissions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/accounts/{account_id}/one/integrations/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationGetResponse,
        )

    def pause(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationPauseResponse:
        """
        Pauses an integration, stopping all crawlers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template("/accounts/{account_id}/one/integrations/{id}/pause", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationPauseResponse,
        )

    def resume(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationResumeResponse:
        """
        Resumes a paused integration, restarting crawlers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template("/accounts/{account_id}/one/integrations/{id}/resume", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationResumeResponse,
        )


class AsyncIntegrationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIntegrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIntegrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIntegrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIntegrationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        application: Literal[
            "ANTHROPIC",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SLACK",
        ],
        credentials: Dict[str, object],
        name: str,
        auth_method: Optional[str] | Omit = omit,
        dlp_profiles: SequenceNotStr[str] | Omit = omit,
        permissions: SequenceNotStr[str] | Omit = omit,
        use_cases: List[Literal["casb", "ces", "auto_remediation"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationCreateResponse:
        """Creates a new integration for the specified application.

        Integration creation
        with OAuth is not supported by API at the moment. For other auth methods, use
        `GET /v2/applications/{application_id}/credential-guide` to see the required
        credential structure and example payloads for each vendor.

        Args:
          application: Vendor/application slug (e.g., GOOGLE_WORKSPACE).

              - `ANTHROPIC` - ANTHROPIC
              - `BITBUCKET` - BITBUCKET
              - `BOX` - BOX
              - `CONFLUENCE` - CONFLUENCE
              - `DROPBOX` - DROPBOX
              - `GITHUB` - GITHUB
              - `GOOGLE_CLOUD_PLATFORM` - GOOGLE_CLOUD_PLATFORM
              - `GOOGLE_WORKSPACE` - GOOGLE_WORKSPACE
              - `JIRA` - JIRA
              - `MICROSOFT_INTERNAL` - MICROSOFT_INTERNAL
              - `OPENAI` - OPENAI
              - `SALESFORCE` - SALESFORCE
              - `SLACK` - SLACK

          credentials: Credentials for the integration.

          name: Name of the integration.

          auth_method: Authentication method slug (uses default if omitted).

          dlp_profiles: List of DLP profile IDs to associate.

          permissions: List of permission scopes (uses policy defaults if empty).

          use_cases: List of use case or feature slugs to enroll (e.g., ['casb', 'ces',
              'auto_remediation']).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/one/integrations", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "application": application,
                    "credentials": credentials,
                    "name": name,
                    "auth_method": auth_method,
                    "dlp_profiles": dlp_profiles,
                    "permissions": permissions,
                    "use_cases": use_cases,
                },
                integration_create_params.IntegrationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationCreateResponse,
        )

    async def update(
        self,
        id: str,
        *,
        account_id: str,
        credentials: Dict[str, object] | Omit = omit,
        dlp_profiles: SequenceNotStr[str] | Omit = omit,
        name: str | Omit = omit,
        permissions: SequenceNotStr[str] | Omit = omit,
        use_cases: List[Literal["casb", "ces", "auto_remediation"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationUpdateResponse:
        """
        Updates an integration's name, permissions, DLP profiles, use cases, or
        credentials.

        Args:
          credentials: Partial credential fields to merge with existing.

          dlp_profiles: List of DLP profile IDs to associate with the integration.

          name: Name of the integration.

          permissions: List of permission scopes granted to the integration.

          use_cases: List of use case or feature slugs to enroll (e.g., ['casb', 'ces',
              'auto_remediation']).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/one/integrations/{id}", account_id=account_id, id=id),
            body=await async_maybe_transform(
                {
                    "credentials": credentials,
                    "dlp_profiles": dlp_profiles,
                    "name": name,
                    "permissions": permissions,
                    "use_cases": use_cases,
                },
                integration_update_params.IntegrationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationUpdateResponse,
        )

    async def list(
        self,
        *,
        account_id: str,
        application: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        dlp_enabled: bool | Omit = omit,
        order: Literal["application", "created", "name", "status"] | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        search: str | Omit = omit,
        status: Literal["Healthy", "Initializing", "Offline", "Unhealthy"] | Omit = omit,
        use_cases: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Returns a paginated list of integrations for the account.

        Args:
          application: Filter by application/vendor (e.g., GOOGLE_WORKSPACE, MICROSOFT_INTERNAL).

          direction: Direction to order results.

          dlp_enabled: Filter by DLP enabled status (true/false).

          order: Field to order results by.

          page: Page number within the paginated result set.

          page_size: Number of results per page.

          search: Search integrations by name or application.

          status: Filter by integration status.

          use_cases: Filter by enabled use cases (e.g., casb, ces). Matches integrations enrolled in
              any of the specified values. Can be specified multiple times.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/one/integrations", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "application": application,
                        "direction": direction,
                        "dlp_enabled": dlp_enabled,
                        "order": order,
                        "page": page,
                        "page_size": page_size,
                        "search": search,
                        "status": status,
                        "use_cases": use_cases,
                    },
                    integration_list_params.IntegrationListParams,
                ),
            ),
            cast_to=object,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an integration by soft-deleting it.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/accounts/{account_id}/one/integrations/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationGetResponse:
        """
        Returns full integration details including use cases and permissions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/one/integrations/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationGetResponse,
        )

    async def pause(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationPauseResponse:
        """
        Pauses an integration, stopping all crawlers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/one/integrations/{id}/pause", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationPauseResponse,
        )

    async def resume(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationResumeResponse:
        """
        Resumes a paused integration, restarting crawlers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/one/integrations/{id}/resume", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationResumeResponse,
        )


class IntegrationsResourceWithRawResponse:
    def __init__(self, integrations: IntegrationsResource) -> None:
        self._integrations = integrations

        self.create = to_raw_response_wrapper(
            integrations.create,
        )
        self.update = to_raw_response_wrapper(
            integrations.update,
        )
        self.list = to_raw_response_wrapper(
            integrations.list,
        )
        self.delete = to_raw_response_wrapper(
            integrations.delete,
        )
        self.get = to_raw_response_wrapper(
            integrations.get,
        )
        self.pause = to_raw_response_wrapper(
            integrations.pause,
        )
        self.resume = to_raw_response_wrapper(
            integrations.resume,
        )


class AsyncIntegrationsResourceWithRawResponse:
    def __init__(self, integrations: AsyncIntegrationsResource) -> None:
        self._integrations = integrations

        self.create = async_to_raw_response_wrapper(
            integrations.create,
        )
        self.update = async_to_raw_response_wrapper(
            integrations.update,
        )
        self.list = async_to_raw_response_wrapper(
            integrations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            integrations.delete,
        )
        self.get = async_to_raw_response_wrapper(
            integrations.get,
        )
        self.pause = async_to_raw_response_wrapper(
            integrations.pause,
        )
        self.resume = async_to_raw_response_wrapper(
            integrations.resume,
        )


class IntegrationsResourceWithStreamingResponse:
    def __init__(self, integrations: IntegrationsResource) -> None:
        self._integrations = integrations

        self.create = to_streamed_response_wrapper(
            integrations.create,
        )
        self.update = to_streamed_response_wrapper(
            integrations.update,
        )
        self.list = to_streamed_response_wrapper(
            integrations.list,
        )
        self.delete = to_streamed_response_wrapper(
            integrations.delete,
        )
        self.get = to_streamed_response_wrapper(
            integrations.get,
        )
        self.pause = to_streamed_response_wrapper(
            integrations.pause,
        )
        self.resume = to_streamed_response_wrapper(
            integrations.resume,
        )


class AsyncIntegrationsResourceWithStreamingResponse:
    def __init__(self, integrations: AsyncIntegrationsResource) -> None:
        self._integrations = integrations

        self.create = async_to_streamed_response_wrapper(
            integrations.create,
        )
        self.update = async_to_streamed_response_wrapper(
            integrations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            integrations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            integrations.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            integrations.get,
        )
        self.pause = async_to_streamed_response_wrapper(
            integrations.pause,
        )
        self.resume = async_to_streamed_response_wrapper(
            integrations.resume,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["AuditProductCategoriesResponse", "Product"]


class Product(BaseModel):
    """A resource product within a product category."""

    label: Optional[str] = None
    """A human-readable label for the product."""

    value: Optional[str] = None
    """The resource_product value that the product category expands to."""


class AuditProductCategoriesResponse(BaseModel):
    """A predefined product category and the resource products it expands to."""

    label: Optional[str] = None
    """A human-readable label for the product category."""

    products: Optional[List[Product]] = None
    """The resource products that the product category expands to."""

    value: Optional[str] = None
    """The product category identifier used with the product_category filter."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["AggregateListResponse", "Aggregation"]


class Aggregation(BaseModel):
    count: float
    """Number of indicators for this aggregation"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[str]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[str]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[str]]


class AggregateListResponse(BaseModel):
    aggregate_by: str = FieldInfo(alias="aggregateBy")
    """Column(s) that were aggregated by"""

    aggregations: List[Aggregation]
    """Array of aggregation results with dynamic fields based on aggregateBy columns"""

    failed_datasets: float = FieldInfo(alias="failedDatasets")
    """Number of datasets whose aggregation failed and were excluded from the result"""

    total: float
    """
    Total count in the aggregation: indicator rows when measure=indicators, or
    linked-event rows when measure=relationships
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["PrefixBindingDeleteResponse", "Message", "Error"]


class Message(BaseModel):
    code: int

    message: str

    error_chain: Optional[List[object]] = None
    """
    Optional upstream error context for APIv4 errors that wrap downstream service
    failures.
    """


class Error(BaseModel):
    code: int

    message: str

    error_chain: Optional[List[object]] = None
    """
    Optional upstream error context for APIv4 errors that wrap downstream service
    failures.
    """


class PrefixBindingDeleteResponse(BaseModel):
    messages: List[Message]

    success: bool

    errors: Optional[List[Error]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AccessibilityTreeCreateResponse", "AccessibilityTree"]


class AccessibilityTree(BaseModel):
    """Accessibility tree node"""

    role: str

    autocomplete: Optional[str] = None

    checked: Union[bool, Literal["mixed"], None] = None

    children: Optional[List[object]] = None

    description: Optional[str] = None

    disabled: Optional[bool] = None

    expanded: Optional[bool] = None

    focused: Optional[bool] = None

    haspopup: Optional[str] = None

    invalid: Optional[str] = None

    keyshortcuts: Optional[str] = None

    level: Optional[float] = None

    modal: Optional[bool] = None

    multiline: Optional[bool] = None

    multiselectable: Optional[bool] = None

    name: Optional[str] = None

    orientation: Optional[str] = None

    pressed: Union[bool, Literal["mixed"], None] = None

    readonly: Optional[bool] = None

    required: Optional[bool] = None

    roledescription: Optional[str] = None

    selected: Optional[bool] = None

    value: Union[str, float, None] = None

    valuemax: Optional[float] = None

    valuemin: Optional[float] = None

    valuetext: Optional[str] = None


class AccessibilityTreeCreateResponse(BaseModel):
    accessibility_tree: Optional[AccessibilityTree] = FieldInfo(alias="accessibilityTree", default=None)
    """Accessibility tree node"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .catalog import (
    CatalogResource,
    AsyncCatalogResource,
    CatalogResourceWithRawResponse,
    AsyncCatalogResourceWithRawResponse,
    CatalogResourceWithStreamingResponse,
    AsyncCatalogResourceWithStreamingResponse,
)
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)

__all__ = [
    "CatalogResource",
    "AsyncCatalogResource",
    "CatalogResourceWithRawResponse",
    "AsyncCatalogResourceWithRawResponse",
    "CatalogResourceWithStreamingResponse",
    "AsyncCatalogResourceWithStreamingResponse",
    "CategoriesResource",
    "AsyncCategoriesResource",
    "CategoriesResourceWithRawResponse",
    "AsyncCategoriesResourceWithRawResponse",
    "CategoriesResourceWithStreamingResponse",
    "AsyncCategoriesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from ...._models import BaseModel

__all__ = ["FindingTimeseriesResponse"]


class FindingTimeseriesResponse(BaseModel):
    """Merged CASB and CDE findings timeseries result."""

    slots: List[Dict[str, object]]
    """Contains time-bucketed result rows.

    Each slot includes a `timestamp` plus `content` and `posture` maps with `cloud`
    and `saas` keys.
    """

    resolution: Optional[str] = None
    """Always null for this endpoint."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ......_types import Body, Query, Headers, NotGiven, not_given
from ......_utils import path_template
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_base_client import make_request_options
from ......types.cloudforce_one.threat_events.indicators.by_dataset.tag_list_response import TagListResponse

__all__ = ["TagsResource", "AsyncTagsResource"]


class TagsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TagsResourceWithStreamingResponse(self)

    def list(
        self,
        dataset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagListResponse:
        """Returns all mirrored tags from the indicator dataset (DO mirror table).

        No
        pagination.

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}/indicators/tags",
                account_id=account_id,
                dataset_id=dataset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagListResponse,
        )


class AsyncTagsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTagsResourceWithStreamingResponse(self)

    async def list(
        self,
        dataset_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagListResponse:
        """Returns all mirrored tags from the indicator dataset (DO mirror table).

        No
        pagination.

        Args:
          account_id: Account ID.

          dataset_id: Dataset ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/dataset/{dataset_id}/indicators/tags",
                account_id=account_id,
                dataset_id=dataset_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagListResponse,
        )


class TagsResourceWithRawResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.list = to_raw_response_wrapper(
            tags.list,
        )


class AsyncTagsResourceWithRawResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.list = async_to_raw_response_wrapper(
            tags.list,
        )


class TagsResourceWithStreamingResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.list = to_streamed_response_wrapper(
            tags.list,
        )


class AsyncTagsResourceWithStreamingResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.list = async_to_streamed_response_wrapper(
            tags.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .robots import (
    RobotsResource,
    AsyncRobotsResource,
    RobotsResourceWithRawResponse,
    AsyncRobotsResourceWithRawResponse,
    RobotsResourceWithStreamingResponse,
    AsyncRobotsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["AIAuditResource", "AsyncAIAuditResource"]


class AIAuditResource(SyncAPIResource):
    @cached_property
    def robots(self) -> RobotsResource:
        return RobotsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AIAuditResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AIAuditResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AIAuditResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AIAuditResourceWithStreamingResponse(self)


class AsyncAIAuditResource(AsyncAPIResource):
    @cached_property
    def robots(self) -> AsyncRobotsResource:
        return AsyncRobotsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAIAuditResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAIAuditResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAIAuditResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAIAuditResourceWithStreamingResponse(self)


class AIAuditResourceWithRawResponse:
    def __init__(self, ai_audit: AIAuditResource) -> None:
        self._ai_audit = ai_audit

    @cached_property
    def robots(self) -> RobotsResourceWithRawResponse:
        return RobotsResourceWithRawResponse(self._ai_audit.robots)


class AsyncAIAuditResourceWithRawResponse:
    def __init__(self, ai_audit: AsyncAIAuditResource) -> None:
        self._ai_audit = ai_audit

    @cached_property
    def robots(self) -> AsyncRobotsResourceWithRawResponse:
        return AsyncRobotsResourceWithRawResponse(self._ai_audit.robots)


class AIAuditResourceWithStreamingResponse:
    def __init__(self, ai_audit: AIAuditResource) -> None:
        self._ai_audit = ai_audit

    @cached_property
    def robots(self) -> RobotsResourceWithStreamingResponse:
        return RobotsResourceWithStreamingResponse(self._ai_audit.robots)


class AsyncAIAuditResourceWithStreamingResponse:
    def __init__(self, ai_audit: AsyncAIAuditResource) -> None:
        self._ai_audit = ai_audit

    @cached_property
    def robots(self) -> AsyncRobotsResourceWithStreamingResponse:
        return AsyncRobotsResourceWithStreamingResponse(self._ai_audit.robots)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.analytics_query.data_security import content_finding_top_n_params
from ....types.analytics_query.data_security.content_finding_top_n_response import ContentFindingTopNResponse

__all__ = ["ContentFindingsResource", "AsyncContentFindingsResource"]


class ContentFindingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContentFindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ContentFindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContentFindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ContentFindingsResourceWithStreamingResponse(self)

    def top_n(
        self,
        *,
        account_id: str,
        filters: Iterable[content_finding_top_n_params.Filter],
        from_: Union[str, datetime],
        n: int,
        to: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[ContentFindingTopNResponse]:
        """
        Returns the top N integrations ranked by total content findings.

        Args:
          filters: Filters to apply. `findingType = content` is applied automatically for CASB
              data.

          from_: Start of the query time range (inclusive). RFC3339.

          n: Maximum number of integrations to return.

          to: End of the query time range (exclusive). RFC3339.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/analytics/query/data-security/content-findings/top-n", account_id=account_id
            ),
            page=SyncSinglePage[ContentFindingTopNResponse],
            body=maybe_transform(
                {
                    "filters": filters,
                    "from_": from_,
                    "n": n,
                    "to": to,
                },
                content_finding_top_n_params.ContentFindingTopNParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ContentFindingTopNResponse,
            method="post",
        )


class AsyncContentFindingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContentFindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContentFindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContentFindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncContentFindingsResourceWithStreamingResponse(self)

    def top_n(
        self,
        *,
        account_id: str,
        filters: Iterable[content_finding_top_n_params.Filter],
        from_: Union[str, datetime],
        n: int,
        to: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ContentFindingTopNResponse, AsyncSinglePage[ContentFindingTopNResponse]]:
        """
        Returns the top N integrations ranked by total content findings.

        Args:
          filters: Filters to apply. `findingType = content` is applied automatically for CASB
              data.

          from_: Start of the query time range (inclusive). RFC3339.

          n: Maximum number of integrations to return.

          to: End of the query time range (exclusive). RFC3339.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/analytics/query/data-security/content-findings/top-n", account_id=account_id
            ),
            page=AsyncSinglePage[ContentFindingTopNResponse],
            body=maybe_transform(
                {
                    "filters": filters,
                    "from_": from_,
                    "n": n,
                    "to": to,
                },
                content_finding_top_n_params.ContentFindingTopNParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ContentFindingTopNResponse,
            method="post",
        )


class ContentFindingsResourceWithRawResponse:
    def __init__(self, content_findings: ContentFindingsResource) -> None:
        self._content_findings = content_findings

        self.top_n = to_raw_response_wrapper(
            content_findings.top_n,
        )


class AsyncContentFindingsResourceWithRawResponse:
    def __init__(self, content_findings: AsyncContentFindingsResource) -> None:
        self._content_findings = content_findings

        self.top_n = async_to_raw_response_wrapper(
            content_findings.top_n,
        )


class ContentFindingsResourceWithStreamingResponse:
    def __init__(self, content_findings: ContentFindingsResource) -> None:
        self._content_findings = content_findings

        self.top_n = to_streamed_response_wrapper(
            content_findings.top_n,
        )


class AsyncContentFindingsResourceWithStreamingResponse:
    def __init__(self, content_findings: AsyncContentFindingsResource) -> None:
        self._content_findings = content_findings

        self.top_n = async_to_streamed_response_wrapper(
            content_findings.top_n,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .isps import (
    ISPsResource,
    AsyncISPsResource,
    ISPsResourceWithRawResponse,
    AsyncISPsResourceWithRawResponse,
    ISPsResourceWithStreamingResponse,
    AsyncISPsResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["DevicesResource", "AsyncDevicesResource"]


class DevicesResource(SyncAPIResource):
    @cached_property
    def isps(self) -> ISPsResource:
        return ISPsResource(self._client)

    @cached_property
    def with_raw_response(self) -> DevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DevicesResourceWithStreamingResponse(self)


class AsyncDevicesResource(AsyncAPIResource):
    @cached_property
    def isps(self) -> AsyncISPsResource:
        return AsyncISPsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDevicesResourceWithStreamingResponse(self)


class DevicesResourceWithRawResponse:
    def __init__(self, devices: DevicesResource) -> None:
        self._devices = devices

    @cached_property
    def isps(self) -> ISPsResourceWithRawResponse:
        return ISPsResourceWithRawResponse(self._devices.isps)


class AsyncDevicesResourceWithRawResponse:
    def __init__(self, devices: AsyncDevicesResource) -> None:
        self._devices = devices

    @cached_property
    def isps(self) -> AsyncISPsResourceWithRawResponse:
        return AsyncISPsResourceWithRawResponse(self._devices.isps)


class DevicesResourceWithStreamingResponse:
    def __init__(self, devices: DevicesResource) -> None:
        self._devices = devices

    @cached_property
    def isps(self) -> ISPsResourceWithStreamingResponse:
        return ISPsResourceWithStreamingResponse(self._devices.isps)


class AsyncDevicesResourceWithStreamingResponse:
    def __init__(self, devices: AsyncDevicesResource) -> None:
        self._devices = devices

    @cached_property
    def isps(self) -> AsyncISPsResourceWithStreamingResponse:
        return AsyncISPsResourceWithStreamingResponse(self._devices.isps)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tags import (
    TagsResource,
    AsyncTagsResource,
    TagsResourceWithRawResponse,
    AsyncTagsResourceWithRawResponse,
    TagsResourceWithStreamingResponse,
    AsyncTagsResourceWithStreamingResponse,
)
from .by_dataset import (
    ByDatasetResource,
    AsyncByDatasetResource,
    ByDatasetResourceWithRawResponse,
    AsyncByDatasetResourceWithRawResponse,
    ByDatasetResourceWithStreamingResponse,
    AsyncByDatasetResourceWithStreamingResponse,
)

__all__ = [
    "TagsResource",
    "AsyncTagsResource",
    "TagsResourceWithRawResponse",
    "AsyncTagsResourceWithRawResponse",
    "TagsResourceWithStreamingResponse",
    "AsyncTagsResourceWithStreamingResponse",
    "ByDatasetResource",
    "AsyncByDatasetResource",
    "ByDatasetResourceWithRawResponse",
    "AsyncByDatasetResourceWithRawResponse",
    "ByDatasetResourceWithStreamingResponse",
    "AsyncByDatasetResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .flags import (
    FlagsResource,
    AsyncFlagsResource,
    FlagsResourceWithRawResponse,
    AsyncFlagsResourceWithRawResponse,
    FlagsResourceWithStreamingResponse,
    AsyncFlagsResourceWithStreamingResponse,
)
from .changelog import (
    ChangelogResource,
    AsyncChangelogResource,
    ChangelogResourceWithRawResponse,
    AsyncChangelogResourceWithRawResponse,
    ChangelogResourceWithStreamingResponse,
    AsyncChangelogResourceWithStreamingResponse,
)

__all__ = [
    "ChangelogResource",
    "AsyncChangelogResource",
    "ChangelogResourceWithRawResponse",
    "AsyncChangelogResourceWithRawResponse",
    "ChangelogResourceWithStreamingResponse",
    "AsyncChangelogResourceWithStreamingResponse",
    "FlagsResource",
    "AsyncFlagsResource",
    "FlagsResourceWithRawResponse",
    "AsyncFlagsResourceWithRawResponse",
    "FlagsResourceWithStreamingResponse",
    "AsyncFlagsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_security.settings import (
    url_ignore_pattern_edit_params,
    url_ignore_pattern_list_params,
    url_ignore_pattern_create_params,
)
from ....types.email_security.settings.url_ignore_pattern_get_response import URLIgnorePatternGetResponse
from ....types.email_security.settings.url_ignore_pattern_edit_response import URLIgnorePatternEditResponse
from ....types.email_security.settings.url_ignore_pattern_list_response import URLIgnorePatternListResponse
from ....types.email_security.settings.url_ignore_pattern_create_response import URLIgnorePatternCreateResponse
from ....types.email_security.settings.url_ignore_pattern_delete_response import URLIgnorePatternDeleteResponse

__all__ = ["URLIgnorePatternsResource", "AsyncURLIgnorePatternsResource"]


class URLIgnorePatternsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> URLIgnorePatternsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return URLIgnorePatternsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> URLIgnorePatternsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return URLIgnorePatternsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        pattern: str,
        comments: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternCreateResponse]:
        """Creates a new URL rewrite ignore pattern.

        URLs matching this pattern will not be
        rewritten.

        Args:
          account_id: Identifier.

          pattern: Regular expression matching URLs that should not be rewritten.

          comments: Optional note describing the reason for the ignore pattern.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email-security/settings/url_ignore_patterns", account_id=account_id),
            body=maybe_transform(
                {
                    "pattern": pattern,
                    "comments": comments,
                },
                url_ignore_pattern_create_params.URLIgnorePatternCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternCreateResponse]], ResultWrapper[URLIgnorePatternCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[URLIgnorePatternListResponse]:
        """Returns a paginated list of URL rewrite ignore patterns for the account.

        URLs
        matching these patterns will not be rewritten.

        Args:
          account_id: Identifier.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/settings/url_ignore_patterns", account_id=account_id),
            page=SyncV4PagePaginationArray[URLIgnorePatternListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    url_ignore_pattern_list_params.URLIgnorePatternListParams,
                ),
            ),
            model=URLIgnorePatternListResponse,
        )

    def delete(
        self,
        pattern_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternDeleteResponse]:
        """Removes a URL rewrite ignore pattern.

        After deletion, URLs matching this pattern
        will be rewritten again.

        Args:
          account_id: Identifier.

          pattern_id: URL ignore pattern identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pattern_id:
            raise ValueError(f"Expected a non-empty value for `pattern_id` but received {pattern_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/url_ignore_patterns/{pattern_id}",
                account_id=account_id,
                pattern_id=pattern_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternDeleteResponse]], ResultWrapper[URLIgnorePatternDeleteResponse]),
        )

    def edit(
        self,
        pattern_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        pattern: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternEditResponse]:
        """Updates an existing URL rewrite ignore pattern.

        Only provided fields will be
        modified.

        Args:
          account_id: Identifier.

          pattern_id: URL ignore pattern identifier

          comments: Optional note describing the reason for the ignore pattern.

          pattern: Regular expression matching URLs that should not be rewritten.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pattern_id:
            raise ValueError(f"Expected a non-empty value for `pattern_id` but received {pattern_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/url_ignore_patterns/{pattern_id}",
                account_id=account_id,
                pattern_id=pattern_id,
            ),
            body=maybe_transform(
                {
                    "comments": comments,
                    "pattern": pattern,
                },
                url_ignore_pattern_edit_params.URLIgnorePatternEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternEditResponse]], ResultWrapper[URLIgnorePatternEditResponse]),
        )

    def get(
        self,
        pattern_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternGetResponse]:
        """
        Returns a single URL rewrite ignore pattern by its identifier.

        Args:
          account_id: Identifier.

          pattern_id: URL ignore pattern identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pattern_id:
            raise ValueError(f"Expected a non-empty value for `pattern_id` but received {pattern_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/url_ignore_patterns/{pattern_id}",
                account_id=account_id,
                pattern_id=pattern_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternGetResponse]], ResultWrapper[URLIgnorePatternGetResponse]),
        )


class AsyncURLIgnorePatternsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncURLIgnorePatternsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncURLIgnorePatternsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncURLIgnorePatternsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncURLIgnorePatternsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        pattern: str,
        comments: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternCreateResponse]:
        """Creates a new URL rewrite ignore pattern.

        URLs matching this pattern will not be
        rewritten.

        Args:
          account_id: Identifier.

          pattern: Regular expression matching URLs that should not be rewritten.

          comments: Optional note describing the reason for the ignore pattern.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email-security/settings/url_ignore_patterns", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "pattern": pattern,
                    "comments": comments,
                },
                url_ignore_pattern_create_params.URLIgnorePatternCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternCreateResponse]], ResultWrapper[URLIgnorePatternCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[URLIgnorePatternListResponse, AsyncV4PagePaginationArray[URLIgnorePatternListResponse]]:
        """Returns a paginated list of URL rewrite ignore patterns for the account.

        URLs
        matching these patterns will not be rewritten.

        Args:
          account_id: Identifier.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/settings/url_ignore_patterns", account_id=account_id),
            page=AsyncV4PagePaginationArray[URLIgnorePatternListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    url_ignore_pattern_list_params.URLIgnorePatternListParams,
                ),
            ),
            model=URLIgnorePatternListResponse,
        )

    async def delete(
        self,
        pattern_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternDeleteResponse]:
        """Removes a URL rewrite ignore pattern.

        After deletion, URLs matching this pattern
        will be rewritten again.

        Args:
          account_id: Identifier.

          pattern_id: URL ignore pattern identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pattern_id:
            raise ValueError(f"Expected a non-empty value for `pattern_id` but received {pattern_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/url_ignore_patterns/{pattern_id}",
                account_id=account_id,
                pattern_id=pattern_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternDeleteResponse]], ResultWrapper[URLIgnorePatternDeleteResponse]),
        )

    async def edit(
        self,
        pattern_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        pattern: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternEditResponse]:
        """Updates an existing URL rewrite ignore pattern.

        Only provided fields will be
        modified.

        Args:
          account_id: Identifier.

          pattern_id: URL ignore pattern identifier

          comments: Optional note describing the reason for the ignore pattern.

          pattern: Regular expression matching URLs that should not be rewritten.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pattern_id:
            raise ValueError(f"Expected a non-empty value for `pattern_id` but received {pattern_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/url_ignore_patterns/{pattern_id}",
                account_id=account_id,
                pattern_id=pattern_id,
            ),
            body=await async_maybe_transform(
                {
                    "comments": comments,
                    "pattern": pattern,
                },
                url_ignore_pattern_edit_params.URLIgnorePatternEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternEditResponse]], ResultWrapper[URLIgnorePatternEditResponse]),
        )

    async def get(
        self,
        pattern_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[URLIgnorePatternGetResponse]:
        """
        Returns a single URL rewrite ignore pattern by its identifier.

        Args:
          account_id: Identifier.

          pattern_id: URL ignore pattern identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pattern_id:
            raise ValueError(f"Expected a non-empty value for `pattern_id` but received {pattern_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/url_ignore_patterns/{pattern_id}",
                account_id=account_id,
                pattern_id=pattern_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[URLIgnorePatternGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[URLIgnorePatternGetResponse]], ResultWrapper[URLIgnorePatternGetResponse]),
        )


class URLIgnorePatternsResourceWithRawResponse:
    def __init__(self, url_ignore_patterns: URLIgnorePatternsResource) -> None:
        self._url_ignore_patterns = url_ignore_patterns

        self.create = to_raw_response_wrapper(
            url_ignore_patterns.create,
        )
        self.list = to_raw_response_wrapper(
            url_ignore_patterns.list,
        )
        self.delete = to_raw_response_wrapper(
            url_ignore_patterns.delete,
        )
        self.edit = to_raw_response_wrapper(
            url_ignore_patterns.edit,
        )
        self.get = to_raw_response_wrapper(
            url_ignore_patterns.get,
        )


class AsyncURLIgnorePatternsResourceWithRawResponse:
    def __init__(self, url_ignore_patterns: AsyncURLIgnorePatternsResource) -> None:
        self._url_ignore_patterns = url_ignore_patterns

        self.create = async_to_raw_response_wrapper(
            url_ignore_patterns.create,
        )
        self.list = async_to_raw_response_wrapper(
            url_ignore_patterns.list,
        )
        self.delete = async_to_raw_response_wrapper(
            url_ignore_patterns.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            url_ignore_patterns.edit,
        )
        self.get = async_to_raw_response_wrapper(
            url_ignore_patterns.get,
        )


class URLIgnorePatternsResourceWithStreamingResponse:
    def __init__(self, url_ignore_patterns: URLIgnorePatternsResource) -> None:
        self._url_ignore_patterns = url_ignore_patterns

        self.create = to_streamed_response_wrapper(
            url_ignore_patterns.create,
        )
        self.list = to_streamed_response_wrapper(
            url_ignore_patterns.list,
        )
        self.delete = to_streamed_response_wrapper(
            url_ignore_patterns.delete,
        )
        self.edit = to_streamed_response_wrapper(
            url_ignore_patterns.edit,
        )
        self.get = to_streamed_response_wrapper(
            url_ignore_patterns.get,
        )


class AsyncURLIgnorePatternsResourceWithStreamingResponse:
    def __init__(self, url_ignore_patterns: AsyncURLIgnorePatternsResource) -> None:
        self._url_ignore_patterns = url_ignore_patterns

        self.create = async_to_streamed_response_wrapper(
            url_ignore_patterns.create,
        )
        self.list = async_to_streamed_response_wrapper(
            url_ignore_patterns.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            url_ignore_patterns.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            url_ignore_patterns.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            url_ignore_patterns.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["FindingExportResponse"]


class FindingExportResponse(BaseModel):
    """Information about an export job."""

    id: str
    """Unique identifier for the export job."""

    status: Literal["Pending", "Success", "Failure", "Rescheduled", "In-Progress"]
    """Status of an export job."""

    type: Literal["finding", "findingInstance", "content", "remediationJob"]
    """Type of export job."""

    user_id: str
    """ID of the export-requesting user."""

    download_url: Optional[str] = None
    """
    The URL by which the successfully created export can be downloaded by the end
    users.
    """

    errors: Optional[str] = None
    """Contains information on errors which may have occurred during export creation."""

    file_name: Optional[str] = None
    """The base name of the file that is/was generated by the export job."""

    file_path: Optional[str] = None
    """
    The full path of the file that is stored within external storage (currently R2).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["FindingExportParams", "Order"]


class FindingExportParams(TypedDict, total=False):
    account_id: Required[str]

    ignored: bool
    """Filter for only the ignored findings. Set to false to only see active items."""

    integration_id: SequenceNotStr[str]
    """Filter by multiple integration IDs."""

    max_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view findings that occurred on or before the affliction date.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    min_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view findings that occurred on or after the affliction date.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    orders: Iterable[Order]
    """Which fields to use when ordering the findings."""

    product: Optional[Literal["SaaS", "Cloud"]]
    """Filter by finding's category product."""

    search: str
    """A search term."""

    severities: List[Literal["CRITICAL", "HIGH", "MEDIUM", "LOW"]]
    """Filter by severity levels."""

    vendors: List[
        Literal[
            "ANTHROPIC",
            "AWS",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SERVICENOW",
            "SLACK",
        ]
    ]
    """Filter by vendor types."""


class Order(TypedDict, total=False):
    """Order specification for finding exports."""

    direction: Required[Literal["asc", "desc"]]
    """Sort direction."""

    name: Required[Literal["instance_count", "finding.name", "integration.name", "latest_affliction_date", "severity"]]
    """Which field to use when ordering the findings."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["TokenCreateParams"]


class TokenCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account identifier."""

    operations: Required[List[Literal["publish", "subscribe"]]]
    """Non-empty subset of the V1 roles the token is allowed to perform.

    Signed into the token.
    """

    expires: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Optional expiry (RFC 3339).

    Defaults to 1 year from creation; rejected if more than 1 year in the future.
    """

    label: str
    """Optional, customer-set label."""
